#!perl

use threads;
use threads::shared;
use FindBin qw($Bin);
use lib $Bin;
use Net::NNTP;
use NNML::Server;
use NNML::Config qw($Config);
use NNmirror;

require "$Bin/config.pl";

my $flag : shared = 1;
my $inloop : shared = 0;
my $interval = shift @ARGV;
my $prefixed = "host|port|user|pass";
my $optexp = "group|" . $prefixed;

open(USERS, "<$Bin/users.txt") or die "Cannot open $Bin/users.txt: $!";
while (<USERS>) {
    next if /^$/ or /^#/;
    next if ! /\sadmin/;
    ($admin, $adpass) = split;
}
close(USERS);

print STDERR "start server\n";
my $server = threads->new(\&server);

#my $nntp = new Net::NNTP localhost, Port => $Config->port;
#$nntp->quit;

print STDERR "feed loop\n";
my $feed = threads->new(\&feed_loop, $interval);

$_ = <> if $interval;
$flag = 0 if $inloop;

end_proc($server, $feed);

# end of main

sub feed_loop {
    my $interval = shift;;

    for $s (keys %site) {
	push(@ropt, nnopts($s, 'f')) if ${$site{$s}}{mode} =~ /r/;
	push(@wopt, nnopts($s, 't')) if ${$site{$s}}{mode} =~ /w/;
    }

    while ($flag) {
	my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst)
	    = localtime(time - 3600 * 24);
	my $update = sprintf("%04d%02d%02d", 1900 + $year, ++$mon, $mday);

	$inloop = 1;

	open(DATE, "<$Bin/date.txt");
	$date = (<DATE>)[0];
	close(DATE);
	chomp $date;
	$date = "$update" if !$date;

	call_nnmirror($date, @ropt);
	print STDERR "NNML::Server::unspool\n";
	unspool;
	call_nnmirror($date, @wopt);

	open(DATE, ">$Bin/date.txt");
	print DATE "$update\n";
	close(DATE);

	if ($interval and $flag) {
	    print STDERR "waiting $interval seconds\n";
	    for (1..$interval) {
		sleep 1;
		last if ! $flag;
	    }
	} else {
	    $flag = 0;
	}
    }
}

sub call_nnmirror {
    my $date = shift;
    for $opt (@_) {
	${$opt}{date} = $date;
	my @opts;
	for $o (keys %{$opt}) {
	    push @opts, "$o => " . ${$opt}{$o};
	}
	print STDERR "nnmirror(" . join(", ", @opts) . ")\n";
	nnmirror(%{$opt});
    }
}

sub nnopts {
    my ($s, $ft) = @_;
    my $tf = ($ft eq 't') ? 'f' : 't';
    my %opt;

    $opt{"${tf}host"} = "localhost";
    $opt{"${tf}user"} = $admin;
    $opt{"${tf}pass"} = $adpass;

    for (keys %{$site{$s}}) {
	next if ! /$optexp/;
	$val = ${$site{$s}}{$_};
	s/($prefixed)/$ft$1/;
	$opt{$_} = $val;
    }

    return \%opt;
}

sub end_proc {
    my ($server, $feed) = @_;
    my $nntp = new Net::NNTP localhost, Port => $Config->port;

    $feed->join;

    $nntp->authinfo($admin, $adpass);
    $nntp->command('SHUT');
    $nntp->quit;
    $server->join;
    print STDERR "server finished\n"
}

# end of file
